
#For original data sources, see references in manuscript
#The processed data used for that analysis in Section 10 of the manuscript
#is available in the file 'Faces.rda', within a .zip folder at this link: https://www.dropbox.com/s/8z30wtw8hd1mk7a/FaceData.zip?dl=0
#this .zip folder also includes the original source files and pre-processing code

load('Faces.rda')
#row and column center images
D = dim(X);
MeansDim234=array(dim=D[2:4]);
for(i in 1:D[2]){ 
  for(j in 1:D[3]){
    for(k in 1:D[4]){
      MeansDim234[i,j,k] <- mean(X[,i,j,k])
      X[,i,j,k] <- X[,i,j,k]-MeansDim234[i,j,k]
}}}

#center Y
Ymat = scale(Ymat)
Y=Ymat

##get traning and test sets
##sample of 1000
n = dim(X)[1]
#train = sample.int(n,size=1000)
#save(train,file='train.rda')
#test = sample(c(1:n)[-train],size=1000)
#save(test,file='test.rda')
load('test.rda')
load('train.rda')
Xtest = X[test,,,]
Ytest = Y[test,]
Xtrain = X[train,,,]
Ytrain=Y[train,]


L = length(dim(Xtrain))-1
N=dim(Xtrain)[1]
P = dim(Xtrain)[2:(L+1)]
M = length(dim(Ytrain))-1
Q = dim(Ytrain)[2:(M+1)]

source('~/rrr.R')
source('~/rrrBayes.R')
###Estimate results for grid of lambda, R
library(doParallel)
library(foreach)
cl <- makeCluster(8)
registerDoParallel(cl)
lambda_indVec = c(rep(1,16),rep(2,16),rep(3,16),rep(4,16),rep(5,16),rep(6,16),rep(7,16),rep(8,16))
lambdaVec = c(0,0.1,1,10,100,1000,10000,100000)
Rvec = rep(c(1:16),8)
temp = foreach(j=1:128) %dopar%{ 
  library(MASS)
  lambda_ind = lambda_indVec[j]
  lambda=lambdaVec[lambda_ind]
  R = Rvec[j]
  Results = rrr(Xtrain,Ytrain,lambda=lambda,R=R,annealIter=50,seed=68)
  save(Results, file=paste("Res",j,".rda"))
}

trainSSE = c()
testSSE = c()
##collect training and test SSE
for(j in 1:128){
  load(file=paste("Res",j,".rda"))
  res=Results
  Rest=dim(res$U[[1]])[2]
  Vmat = matrix(nrow=prod(Q),ncol=Rest)
  for(r in 1:Rest){
    Vr = lapply(res$V, function(x) x[,r])
    Vmat[,r] = as.vector(array(apply(expand.grid(Vr), 1, prod), dim=Q))
  }
  Bmat = matrix(nrow=Rest,ncol=prod(P))
  for(r in 1:Rest){
    Ur = lapply(res$U, function(x) x[,r])
    Br = array(apply(expand.grid(Ur), 1, prod), dim=P)  
    Bmat[r,] = array(Br,dim=c(1,prod(P)))
  }
  
  Xmat.tr = array(Xtrain,dim=c(N,prod(P)))
  Yvec.tr = as.vector(Ytrain)
  Ypred.tr = as.vector(Xmat.tr%*%t(Bmat)%*%t(Vmat))
  trainSSE[j] = sum((Yvec.tr-Ypred.tr)^2)/sum(Yvec.tr^2)
  
  Xmat.te = array(Xtest,dim=c(N,prod(P)))
  Yvec.te = as.vector(Ytest)
  Ypred.te = as.vector(Xmat.te%*%t(Bmat)%*%t(Vmat))
  testSSE[j] = sum((Yvec.te-Ypred.te)^2)/sum(Yvec.te^2)
}

##Now train,test SSE for stanard ridge regression (full rank)
XtrainMat <- matrix(Xtrain,nrow=1000,ncol=prod(P))
XtrainMat = XtrainMat[1:1000,]
Ytrain=Ytrain[1:1000,]
XtestMat <- matrix(Xtest,nrow=1000,ncol=prod(P))
Xdesign <- tcrossprod(XtrainMat)
trainSSEf = c()
testSSEf =  c()
for(j in 1:7){
  lambda=lambdaVec[j+1]
  B <- t(XtrainMat)%*%(solve(Xdesign +lambda*diag(N))%*%Ytrain)
  Ytrain.pre <- XtrainMat %*% B
  trainSSEf[j] <- sum((Ytrain.pre-Ytrain)^2)/sum(Ytrain^2)
  Ytest.pre <- XtestMat %*% B
  testSSEf[j] <- sum((Ytest.pre-Ytest)^2)/sum(Ytest^2)
}

###Heatmap:
library(reshape2)
library(ggplot2)
Rank = c(1:16,'FULL')
TestMatThresh=rbind(TestMat[1:16,1:8],c(NA,testSSEf))
TestMatThresh[TestMatThresh>1] = 1
TestTable <- cbind(Rank,data.frame(TestMatThresh))
TestTable <- melt(TestTable, id.vars="Rank")
TestTable=rbind(TestTable)
TestTable$Rank <- factor(TestTable$Rank, levels=unique(as.character(TestTable$Rank)) )
map<- qplot(x=variable, y=Rank, data=TestTable, fill=value, geom="tile") 
map<- map+ theme(axis.text.x = element_text(angle = 90, hjust = 1))
map<- map+ scale_fill_gradient2(labels=c('0.6','0.7','0.8','0.9','>1'),limits=c(0.56,1),low = "blue", mid = "white", high = "red",midpoint=0.80)
map<- map + labs(x='Lambda',fill='SSE') 
map<-map + scale_x_discrete(labels=c("X1" = "0", "X2" = "0.1","X3"="1","X4"="10","X5"="100","X6"="1000","X7"="10000","X8"="100000"))
png(file = "~/Dropbox/ContinuumTensorRegression/Manuscript/Fig_heatmap.png",width=600,height=400)
map
dev.off()

#####Selected model
load(file=paste("Res",127,".rda")) #R=15,lambda=10^5
res=Results
Rest=dim(res$U[[1]])[2]
Vmat = matrix(nrow=prod(Q),ncol=Rest)
for(r in 1:Rest){
  Vr = lapply(res$V, function(x) x[,r])
  Vmat[,r] = as.vector(array(apply(expand.grid(Vr), 1, prod), dim=Q))
}
Bmat = matrix(nrow=Rest,ncol=prod(P))
for(r in 1:Rest){
  Ur = lapply(res$U, function(x) x[,r])
  Br = array(apply(expand.grid(Ur), 1, prod), dim=P)  
  Bmat[r,] = array(Br,dim=c(1,prod(P)))
}

Xmat.te = array(Xtest,dim=c(N,prod(P)))
##Observed vs predicted
Yvec.te = as.vector(Ytest)
Ypred.te = as.vector(Xmat.te%*%t(Bmat)%*%t(Vmat))
layout(matrix(c(1,1),nrow = 1,ncol=1))
png(file='~/Dropbox/ContinuumTensorRegression/Manuscript/ObsVsPre.png',width=800,height=800)
par(mar=c(5.1,5.1,4.1,2.1))
plot(Ypred.te,Yvec.te,xlab='Predicted',ylab='Actual',cex.lab=2,cex.axis=2)
dev.off()


####Run for 5000 posterior samples for selected model
load(file=paste("Res",127,".rda")) #R=15,lambda=10^5
res=Results
X=Xtrain
Y=Ytrain
U=res$U
V=res$V
Xmat.new=XtestMat
Ysamps<-rrrBayes(Xtrain,Ytrain,res$U,res$V,XtestMat,lambda=10^5,R=15,Samples=1000,thin=1)
##run 4 more times in parallel
cl <- makeCluster(4)
registerDoParallel(cl)
temp = foreach(j=1:4) %dopar%{ 
  library(MASS)
  load(file=paste("Res",127,".rda"))
  res=Results
  Ysamps<-rrrBayes(Xtrain,Ytrain,res$U,res$V,XtestMat,lambda=10^5,R=15,Samples=1000,thin=1)
  save(Ysamps,file=paste("Ysamps",j,".rda"))
}

load(file="Ysamps.rda")
YsampsAll = Ysamps
load(file="Ysamps 1 .rda")
YsampsAll = rbind(YsampsAll,Ysamps)
load(file="Ysamps 2 .rda")
YsampsAll = Ysamps
load(file="Ysamps 3 .rda")
YsampsAll = rbind(YsampsAll,Ysamps)
load(file="Ysamps 4 .rda")
YsampsAll = rbind(YsampsAll,Ysamps)
Bounds = apply(YsampsAll, 2, quantile, probs = c(0.025, 0.975),  na.rm = TRUE)
Coverage= sum((Y.new > Bounds[1,]) & (Y.new < Bounds[2,]))/length(Y.new)
Bounds = apply(YsampsAll, 2, quantile, probs = c(0.05, 0.95),  na.rm = TRUE)
Coverage90= sum((Y.new > Bounds[1,]) & (Y.new < Bounds[2,]))/length(Y.new)
Bounds = apply(YsampsAll, 2, quantile, probs = c(0.005, 0.995),  na.rm = TRUE)
Coverage99= sum((Y.new > Bounds[1,]) & (Y.new < Bounds[2,]))/length(Y.new)
rel.length= mean(Bounds[2,]-Bounds[1,])/sd(Y.new) 

##figure for example face
OrigTest = AllImages[test,,,]

J=4
plot.new()
col <- rgb(OrigTest[J,,,1], OrigTest[J,,,2], OrigTest[J,,,3],maxColorValue=255)
dim(col) <- c(90,90)
#library(grid)
grid.raster(col, interpolate=FALSE)

Ytest[4,18] ##smiling
YsampsArray = array(Ysamps,dim=c(dim(Ysamps)[1],1000,72))
hist(YsampsArray[,4,18],xlim=c(-3.5,3.5))

Ytest[4,15] ##glasses
hist(YsampsArray[,4,15],xlim=c(-3.5,3.5))
colMeans(YsampsArray[,4,])

par(mar=c(0,0,0,0))
png(file='Face_fig.png',width=800,height=800)
plot.new()
grid.raster(col, interpolate=FALSE)
dev.off()
par(mar=c(0,0,0,0))
png(file='~/Dropbox/ContinuumTensorRegression/Manuscript/hists.png',width=800,height=800)
layout(matrix(c(1,2,3,4,5),nrow = 5,ncol=1))
hist(YsampsArray[,4,1],xlim=c(-3.5,3.5),main='Male',xlab=NULL,ylab=NULL,yaxt='n',cex.main=2,cex.axis=2) ##male
hist(YsampsArray[,4,18],xlim=c(-3.5,3.5),main='Smiling',xlab=NULL,ylab=NULL,yaxt='n',cex.main=2,cex.axis=2) ##smiling
hist(YsampsArray[,4,6],xlim=c(-3.5,3.5),main='Child',xlab=NULL,ylab=NULL,yaxt='n',cex.main=2,cex.axis=2) ##child
hist(YsampsArray[,4,15],xlim=c(-3.5,3.5),main='Glasses',xlab=NULL,ylab=NULL,yaxt='n',cex.main=2,cex.axis=2) ##glasses
hist(YsampsArray[,4,10],xlim=c(-3.5,3.5),main='Black hair',xlab=NULL,ylab=NULL,yaxt='n',cex.main=2,cex.axis=2) ##black hair
dev.off()






  