
source('~/Revision Code/rrr.R')
source('~/Revision Code/rrrBayes.R')
##run simulation for predictive performance


library(doParallel)
library(foreach)
cl <- makeCluster(10)
registerDoParallel(cl)
q1 = 5
q2 = 10
p1 = 15
p2 = 20
nvec = c(30,120)
s2nvec = c(1,5)
lambdaVec = c(0,0.5,1,5,50)
Seeds = array(30*c(1:(10*2*2*5)),dim=c(30,2,2,5))
for(i in 1:10){
  #  library(MASS)
  s2n_indvec = c(rep(1,5),rep(2,5))
  lambda_indVec = rep(c(1,2,3,4,5),2)
  All = foreach(j=1:10) %dopar%{ 
    library(MASS)
    library(fields)
    s2n_ind=s2n_indvec[j]
    lambda_ind=lambda_indVec[j]
    SSE = array(dim=c(2,2,5,5))
    Iters = array(dim=c(2,2,5,5))
    Coverage = array(dim=c(2,2,5,5))
    rel.length = array(dim=c(2,2,5,5))
    for(n_ind in 1:2){  
      n = nvec[n_ind]
      s2n = s2nvec[s2n_ind]
      for(R in 1:5){
        set.seed(Seeds[i,n_ind,s2n_ind,R])
        Ye = array(rnorm(n*q1*q2),dim=(c(n,q1,q2)))
        X = array(rnorm(n*p1*p2), dim=c(n,p1,p2))
        L = length(dim(X))-1
        N=dim(X)[1]
        P = dim(X)[2:(L+1)]
        M = length(dim(Ye))-1
        Q = dim(Ye)[2:(M+1)]
        Xmat = array(X,dim=c(n,prod(P)))
        Y=Ye
        U = list()
        for(l in 1:L) U[[l]] = matrix(rnorm(P[l]*R),ncol=R)
        V = list()
        for(l in 1:M) V[[l]] = matrix(rnorm(Q[l]*R),ncol=R)
        Vmat.true = matrix(nrow=prod(Q),ncol=R)
        for(r in 1:R){
          Vr = lapply(V, function(x) x[,r])
          Vmat.true[,r] = as.vector(array(apply(expand.grid(Vr), 1, prod), dim=Q))
        }
        
        Bmat.true = matrix(nrow=R,ncol=prod(P))
        for(r in 1:R){
          Ur = lapply(U, function(x) x[,r])
          Br = array(apply(expand.grid(Ur), 1, prod), dim=P)  
          Bmat.true[r,] = array(Br,dim=c(1,prod(P)))
        }
        Ysig = as.vector(Xmat%*%t(Bmat.true)%*%t(Vmat.true))
        Ysig = array(as.vector(Ysig),dim=(c(n,q1,q2)))
        s2n.scale = s2n*sqrt(sum(Ye^2))/sqrt(sum(Ysig^2))
        Y = Ysig*s2n.scale+Ye
        lambda = lambdaVec[lambda_ind]
        Rest=R
        ###Run algorithm from 5 initializations, choose estimate with best residual SSE 
        resList=list()
        resSSE = c()
        for(iter in 1:5){
          resList[[iter]] <- rrr(X,Y,lambda,R=Rest,annealIter=100)
          resSSE[iter] = resList[[iter]]$sse[length(resList[[iter]]$sse)]
        }  
        res = resList[[which.min(resSSE)]]
        Vmat = matrix(nrow=prod(Q),ncol=Rest)
        for(r in 1:Rest){
          Vr = lapply(res$V, function(x) x[,r])
          Vmat[,r] = as.vector(array(apply(expand.grid(Vr), 1, prod), dim=Q))
        }
        Bmat = matrix(nrow=Rest,ncol=prod(P))
        for(r in 1:Rest){
          Ur = lapply(res$U, function(x) x[,r])
          Br = array(apply(expand.grid(Ur), 1, prod), dim=P)  
          Bmat[r,] = array(Br,dim=c(1,prod(P)))
        }
        ##generate test data
        test = 500
        Xmat.new = array(rnorm(test*p1*p2),dim=c(test,p1*p2))
        Ypred = as.vector(Xmat.new%*%t(Bmat)%*%t(Vmat))
        if(R>0){
          Ysig.new = as.vector(Xmat.new%*%t(Bmat.true)%*%t(Vmat.true))
          Y.new = Ysig.new*s2n.scale+rnorm(test*q1*q2)}
        if(R==0){Y.new=rnorm(test*q1*q2)}
        X.new <- array(Xmat.new,dim=c(test,p1,p2))
        Ysamps <- rrrBayes(X,Y,Inits=list(U=res$U,V=res$V),X.new=X.new, lambda=lambda,R=Rest,Samples=100,thin=10)
        Ysamps <- array(Ysamps,dim=c(10,test*q1*q2))
        Bounds = apply(Ysamps, 2, quantile, probs = c(0.025, 0.975),  na.rm = TRUE)
        Coverage[n_ind,s2n_ind,lambda_ind,R] = sum((Y.new > Bounds[1,]) & (Y.new < Bounds[2,]))/length(Y.new)
        rel.length[n_ind,s2n_ind,lambda_ind,R] = mean(Bounds[2,]-Bounds[1,])/sd(Y.new)
        SSE[n_ind,s2n_ind,lambda_ind,R] = sum((Ypred-Y.new)^2)/sum((Y.new)^2)
        Iters[n_ind,s2n_ind,lambda_ind,R]  = length(res$sse)
      }} 
    list(SSE,Iters,rel.length,Coverage) 
  }
  
  save(All, file=paste("~/Revision Code/Sims/FullSimBayesI",i,".rda"))
}
